package user

import (
	domainUser "cls-server/internal/domain/user"
	"cls-server/pkg/logger"
	"cls-server/pkg/util/page"
	"cls-server/pkg/xorm_engine"
	"errors"
	"time"
	"xorm.io/builder"
)

// UserRepositoryORM ORM 实现UserRepository 持久化数据
type UserRepositoryORM struct {
	engine *xorm_engine.Engine
	log    logger.Logger
}

var _ domainUser.UserRepository = (*UserRepositoryORM)(nil)

func NewUserRepositoryORM(db *xorm_engine.Engine, logger logger.New) domainUser.UserRepository {
	return &UserRepositoryORM{
		engine: db,
		log:    logger("panoramic:persistence:user"),
	}
}

func (u *UserRepositoryORM) Save(user *domainUser.User) error {
	if user.Id > 0 {
		_, err := u.engine.Cols("status").Unscoped().Update(user)
		return err
	}
	_, err := u.engine.Insert(user)
	if err != nil {
		u.log.Error(err)
	}
	return err
}

func (u *UserRepositoryORM) GetUserGiftCount(ePhone string) (int, error) {
	giftCount := 0
	_, err := u.engine.Where(builder.Eq{"phone": ePhone}).Cols("gift_count").Table(&domainUser.User{}).Get(&giftCount)
	if err != nil {
		u.log.Error(err.Error())
		return giftCount, err
	}
	return giftCount, nil
}

func (u *UserRepositoryORM) FindByUsername(username string) (*domainUser.User, error) {
	user := &domainUser.User{}
	exist, err := u.engine.Where(builder.Eq{"username": username}).Get(user)
	if err != nil {
		return nil, err
	}
	if !exist {
		return nil, errors.New("用户不存在")
	}
	return user, nil
}

func (u *UserRepositoryORM) FindByID(id uint64) (*domainUser.User, error) {
	user := &domainUser.User{}
	exist, err := u.engine.Where(builder.Eq{"id": id}).Get(user)
	if err != nil {
		return nil, err
	}
	if !exist {
		return nil, errors.New("用户不存在")
	}

	return user, nil
}

func (u *UserRepositoryORM) FindByPhone(phone string) (*domainUser.User, error) {
	user := &domainUser.User{}
	exist, err := u.engine.Where(builder.Eq{"phone": phone}).Get(user)
	if err != nil {
		return nil, err
	}
	if !exist {
		return nil, errors.New("用户不存在")
	}

	return user, nil
}

func (u *UserRepositoryORM) FindAll(page *page.Page, conds []builder.Cond) error {
	err := u.engine.FindAll(page, &domainUser.User{}, builder.And(conds...))
	if err != nil {
		u.log.Error(err)

	}
	return err
}

func (u *UserRepositoryORM) Delete(id uint64) error {
	_, err := u.engine.Where(builder.Eq{"id": id}).Delete(&domainUser.User{})
	if err != nil {
		u.log.Error(err)
	}
	return err
}

func (u *UserRepositoryORM) UpdateLastLogin(id uint64, ip string) error {
	now := time.Now()
	_, err := u.engine.Where(builder.Eq{"id": id}).
		Cols("last_login_time", "last_login_ip").
		Update(&domainUser.User{
			LastLoginTime: &now,
			LastLoginIp:   ip,
		})
	return err
}

func (u *UserRepositoryORM) IsLocked(status int8) bool {
	if status == 0 {
		return false
	} else {
		return true
	}

}
func (u *UserRepositoryORM) UpdateUserGiftCount(id uint64, count int8) error {
	_, err := u.engine.Where(builder.Eq{"id": id}).Cols("gift_count").Update(&domainUser.User{GiftCount: count})
	return err
}
func (u *UserRepositoryORM) UpdateUserInfo(user *domainUser.User) error {
	_, err := u.engine.Update(user)
	return err
}

func (u *UserRepositoryORM) SetPassword(phone string, ePhone string, pwd string) error {
	_, err := u.engine.Where(builder.Eq{"phone": ePhone}).Cols("username", "password").
		Update(&domainUser.User{Username: phone, Password: pwd})
	if err != nil {
		u.log.Error(err)
	}
	return err
}
