import {Observable, of} from 'rxjs';

/**
 * Central class for pagination support. Holds typed collections that can be
 * paginated over.
 *
 * @author Gunnar Hillert
 */
export class Page<T> {
  totalPages: number = 0;
  totalElements: number = 0;

  /**
   * The page number is 0-index-based.
   */
  pageNumber = 0;
  pageSize = 10;
  items: T[] = [];
  filter = {};
  sort = {};
  paginationId = 'pagination-instance';

  public static fromJSON<T>(input: any): Page<T> {
    const page = new Page<T>();
    if (input && input.page) {
      page.pageNumber = input.page.number;
      page.pageSize = input.page.size;
      page.totalElements = input.page.totalElements;
      page.totalPages = input.page.totalPages;
    }
    return page;
  }

  /**
   * Helper method for ngx-pagination. Allows for passing in
   * pagination meta information directly to ngx-pagination
   * by conforming to the {@link PaginationInstance} interface.
   *
   * If using multiple pagination controls on the page,
   * you may need to set the {@link Page#paginationId} to a different
   * value.
   *
   * One important fact is that the returned currentPage number
   * for {@link PaginationInstance} is 1-index-based. The conversion
   * is done implicitly.
   */
  public getPaginationInstance(): PaginationInstance {
    return {
      id: this.paginationId,
      itemsPerPage: this.pageSize,
      currentPage: this.pageNumber + 1,
      totalItems: this.totalElements
    };
  }

  public getItemsAsObservable(): Observable<T[]> {
    return of(this.items);
  }

  public update(page: Page<T>): void {
    this.items.length = 0;
    this.items.push(...page.items);
    this.pageNumber = page.pageNumber;
    this.pageSize = page.pageSize;
    this.totalElements = page.totalElements;
    this.totalPages = page.totalPages;
  }

}

export interface PaginationInstance {
  /**
   * An optional ID for the pagination instance. Only useful if you wish to
   * have more than once instance at a time in a given component.
   */
  id?: string;
  /**
   * The number of items per paginated page.
   */
  itemsPerPage: number;
  /**
   * The current (active) page.
   */
  currentPage: number;
  /**
   * The total number of items in the collection. Only useful when
   * doing server-side paging, where the collection size is limited
   * to a single page returned by the server API.
   *
   * For in-memory paging, this property should not be set, as it
   * will be automatically set to the value of  collection.length.
   */
  totalItems?: number;
}

export function extractData<T>(data: any): Page<T> {
  const items: T[] = data.content;
  const page = new Page<T>();
  page.items = items;
  page.totalElements = data.totalElements;
  page.pageNumber = data.number;
  page.pageSize = data.size;
  page.totalPages = data.totalPages;
  return page;
}
