import {Component, Inject, OnInit} from '@angular/core';
import {UserInfo} from "../../../core/models/user";
import {NzModalModule, NzModalService} from "ng-zorro-antd/modal";
import {UserService} from "../user.service";
import {FormBuilder, FormGroup, FormsModule, ReactiveFormsModule, Validators} from "@angular/forms";
import {CouponType} from "../../../core/models/coupon";
import {getUser} from "../../../core/auth/auth.service";
import {NzFormModule} from "ng-zorro-antd/form";
import {NzInputModule} from "ng-zorro-antd/input";
import {NzSelectModule} from "ng-zorro-antd/select";
import {NzInputNumberModule} from "ng-zorro-antd/input-number";
import {NzDatePickerModule} from "ng-zorro-antd/date-picker";
import {NzButtonModule} from "ng-zorro-antd/button";
import {NzDrawerRef, NZ_DRAWER_DATA} from "ng-zorro-antd/drawer";
import {CommonModule} from "@angular/common";
import { differenceInDays } from 'date-fns';
import { Coupon } from '../../../core/models/coupon';
import { NzMessageService } from 'ng-zorro-antd/message';
import { NzMessageModule } from 'ng-zorro-antd/message';
export interface DrawerData {
  user: UserInfo;
}

@Component({
  selector: 'app-coupon',
  standalone: true,
  imports: [
    CommonModule,
    NzModalModule,
    NzFormModule,
    NzInputModule,
    NzSelectModule,
    NzInputNumberModule,
    NzDatePickerModule,
    NzButtonModule,
    FormsModule,
    ReactiveFormsModule,
    NzMessageModule,
  ],
  templateUrl: './coupon.component.html',
  styleUrl: './coupon.component.scss'
})
export class CouponComponent implements OnInit {
  user!: UserInfo;
  isSaving: boolean = false;
  validateForm!: FormGroup;
  couponType = CouponType;
  
  // 预设日期范围选项
  dateRanges: { [key: string]: Date[] } = {
    '今天': [new Date(), new Date()],
    '本周': [this.getWeekStartDate(), new Date()],
    '本月': [this.getMonthStartDate(), new Date()],
    '一个月': [new Date(), this.addDays(new Date(), 30)],
    '三个月': [new Date(), this.addDays(new Date(), 90)]
  };

  constructor(
    private fb: FormBuilder,
    private modalService: NzModalService,
    private userService: UserService,
    private message: NzMessageService,
    private drawerRef: NzDrawerRef,
    @Inject(NZ_DRAWER_DATA) private data: DrawerData
  ) {
    // 如果能通过注入获取用户数据
    if (this.data && this.data.user) {
      this.user = this.data.user;
    }
    
    this.validateForm = this.fb.group({
      name: [null, [Validators.required]],
      type: [null, [Validators.required]],
      value: [0, [Validators.required, Validators.min(0)]],
      minAmount: [0, [Validators.required, Validators.min(0)]],
      dateRange: [null, [Validators.required]],
      adminId: [null, [Validators.required]],
      userId: [null, [Validators.required]],
    });
  }

  ngOnInit() {
    // 设置用户ID
    if (this.user) {
      this.validateForm.patchValue({
        userId: this.user.uid
      });
    }

    // 获取管理员信息
    getUser().subscribe((admin) => {
      if (admin) {
        this.validateForm.patchValue({
          adminId: admin.id
        });
      }
    });
  }

  // 获取本周开始日期
  private getWeekStartDate(): Date {
    const now = new Date();
    const day = now.getDay();
    const diff = now.getDate() - day + (day === 0 ? -6 : 1); // 如果是周日，减6天；其他天减去对应天数
    return new Date(now.setDate(diff));
  }

  // 获取本月开始日期
  private getMonthStartDate(): Date {
    const now = new Date();
    return new Date(now.getFullYear(), now.getMonth(), 1);
  }

  // 添加天数
  private addDays(date: Date, days: number): Date {
    const result = new Date(date);
    result.setDate(result.getDate() + days);
    return result;
  }

  // 格式化数字显示
  valueFormatter = (value: number): string => {
    return value ? `${value}` : '';
  };

  // 解析数字输入
  valueParser = (value: string): string => {
    return value.replace(/[^\d.]/g, '');
  };

  confirm() {
    if (this.validateForm.valid && this.user) {
      const formValue = this.validateForm.value;
      
      // 获取日期范围
      const dateRange = formValue.dateRange;
      const startTime = dateRange?.[0];
      const endTime = dateRange?.[1];
      
      // 创建确认消息
      const typeText = formValue.type === CouponType.DISCOUNT ? '折扣券' : '满减券';
      const valueText = formValue.type === CouponType.DISCOUNT ? `${formValue.value}折` : `${formValue.value}元`;
      
      this.modalService.create({
        nzTitle: "优惠券发放",
        nzContent: `确认对用户ID【${this.user.uid}】，手机号为【${this.user.phone}】发放${typeText}：${valueText}，最低使用金额：${formValue.minAmount}元`,
        nzOnOk: () => {
          this.giveCoupon()
        }
      });
    } else {
      Object.values(this.validateForm.controls).forEach(control => {
        if (control.invalid) {
          control.markAsTouched();
          control.updateValueAndValidity({ onlySelf: true });
        }
      });
    }
  }

  cancel() {
    this.drawerRef.close();
  }

  giveCoupon() {
    // TODO: 实现优惠券发放逻辑
    // 1. 获取表单数据
    const formValue = this.validateForm.value;
    const dateRange = formValue.dateRange;
    
    // 2. 构建API请求数据
    const couponData = {
      name: formValue.name,
      type: formValue.type,
      value: formValue.value,
      minAmount: formValue.minAmount,
      startTime: dateRange?.[0],
      endTime: dateRange?.[1],
      adminId: formValue.adminId,
      userId: formValue.userId
    };
    
    // 3. 设置加载状态
    this.isSaving = true;
    
    // 4. 在这里调用后端API
    console.log('发放优惠券数据:', couponData);
    this.userService.giveCoupon(couponData as Coupon).subscribe({
      next: (res) => {
        this.message.success('优惠券发放成功');
        this.isSaving = false;
        this.drawerRef.close(true); // 传递true表示操作成功，需要刷新列表
      },
      error: (err) => {
        this.message.error('优惠券发放失败');
        this.isSaving = false;
        this.drawerRef.close(true); // 传递true表示操作成功，需要刷新列表
      }
    });
    
  }
}

