import {ChangeDetectorRef, Component, OnDestroy, OnInit, TemplateRef, ViewChild} from '@angular/core';

import { NzBreadCrumbModule } from 'ng-zorro-antd/breadcrumb';
import { NzPageHeaderModule } from 'ng-zorro-antd/page-header';
import {UserService} from "./user.service";
import {UserInfo} from "../../core/models/user";
import {Page} from "../../core/models/page";
import {NzModalModule, NzModalService} from "ng-zorro-antd/modal";
import {NzMessageService} from "ng-zorro-antd/message";
import {debounceTime, distinctUntilChanged, Subject} from "rxjs";
import {takeUntil} from "rxjs/operators";
import {NzCardModule} from "ng-zorro-antd/card";
import {NzInputModule} from "ng-zorro-antd/input";
import {FormsModule} from "@angular/forms";
import {NzTableModule} from "ng-zorro-antd/table";
import {NzTagModule} from "ng-zorro-antd/tag";
import {NzSwitchModule} from "ng-zorro-antd/switch";
import {NzDropDownModule} from "ng-zorro-antd/dropdown";
import {NzSpaceModule} from "ng-zorro-antd/space";
import {NzButtonModule} from "ng-zorro-antd/button";
import {NzIconDirective} from "ng-zorro-antd/icon";
import {NzRowDirective} from "ng-zorro-antd/grid";
import {NzPopoverDirective} from "ng-zorro-antd/popover";
import {NgFor} from "@angular/common";
import {NzDrawerModule, NzDrawerService} from "ng-zorro-antd/drawer";
import {CouponComponent} from "./coupon/coupon.component";
import {GiftComponent} from "./gift/gift.component";
import {Gift} from "../../core/models/gift";
import { UserDetail } from '../../core/models/admin';
import { getUser } from '../../core/auth/auth.service';
@Component({
  selector: 'app-user',
  standalone: true,
  imports: [NzBreadCrumbModule,
    NzPageHeaderModule,
    NzCardModule,
    NzInputModule,
    FormsModule,
    NzTableModule,
    NzTagModule,
    NzSwitchModule,
    NzDropDownModule,
    NzSpaceModule,
    NzButtonModule,
    NzIconDirective,
    NzRowDirective,
    NzPopoverDirective,
    NzDrawerModule,
    NgFor,
  ],
  templateUrl: './user.component.html'
})
export class UserComponent implements OnInit,OnDestroy{
  pageData: Page<UserInfo> = new Page<UserInfo>();
  searchParams: { [param: string]: any } = {
    page: 0,
    size: 10,
    search_like_username: ''
  };
  onValueChange = new Subject<string>();
  loading = false;
  destroy$ = new Subject();
  admin!:UserDetail  ;
  constructor(private cdr:ChangeDetectorRef,
              private userService:UserService,
              private drawerService:NzDrawerService,
              private message: NzMessageService,
              private modalService: NzModalService,) {
    this.onValueChange.pipe(
      debounceTime(20),
      distinctUntilChanged(),
      takeUntil(this.destroy$)).subscribe(() => {
      this.searchParams['page'] = 0;
      this.cdr.markForCheck();
      this.loadData();
    });
    getUser().subscribe(data=>{
      this.admin = data;
    })
  }
  ngOnInit(): void {
    this.loadData();
  }

  ngOnDestroy(): void {
    this.destroy$.next(null);
    this.destroy$.complete();
  }
  loadData(){
    this.loading = true
    this.cdr.detectChanges();
    this.userService.page(this.searchParams).
    pipe(takeUntil(this.destroy$)).
    subscribe({
      next:data=>{
        this.loading = false
        this.pageData = data
      },
      error:data=>{
        this.loading = false;
        this.message.error("获取用户列表失败！")
      }
    })
  }
  /**
   * Update event from the Paginator Pager
   * @Param page
   */
  changePaginationPage(page: number): void {
    this.searchParams['page'] = page - 1;
    this.cdr.detectChanges();
    this.loadData();
  }

  /**
   * Update event from the Paginator Pager
   * @Param size
   */
  changePaginationSize(size: number): void {
    this.searchParams['page'] = 0;
    this.searchParams['size'] = size;
    this.cdr.detectChanges();
    this.loadData();
  }

  switchEnable(user: UserInfo): void {
    if(user.status) {
      this.userService.enableUser(user.uid).subscribe({
        next:data=>{
          this.message.success("启用用户成功")
        },
        error:data =>{
          this.message.error("启用用户失败")
        }
      })
    } else {
      this.userService.disableUser(user.uid).subscribe({
        next:data=>{
          this.message.success("禁用用户成功")
        },
        error:data =>{
          this.message.error("禁用用户失败")
        }
      })
    }

  }
  giveCoupon(user:UserInfo){
    const drawerRef = this.drawerService.create({
      nzTitle: "优惠券发放",
      nzClosable: false,
      nzMaskClosable: false,
      nzContent: CouponComponent,
      nzSize: "large",
      nzWidth: 800,
      nzData: {
        user: user
      }
    });

    drawerRef.afterClose.subscribe(data => {
      if (data) {
        this.loadData();
      }
    });
  }
  giveGift(user: UserInfo) {
    const modal = this.modalService.create({
      nzTitle: '赠送免费解锁次数',
      nzContent: GiftComponent,
      nzData: {
        user: user
      },
      nzOkText: '确认赠送',
      nzCancelText: '取消',
      nzOnOk: (componentInstance) => {
        const result = componentInstance.submitForm();
        if (result) {
          // 显示二次确认对话框
          return new Promise<boolean>((resolve) => {
            this.modalService.confirm({
              nzTitle: '确认赠送',
              nzContent: `确定要赠送 ${result.unlockCount} 次免费解锁次数给用户 ${user.username || user.phone} 吗？`,
              nzOkText: '确定',
              nzCancelText: '取消',
              nzOnOk: () => {
                // 调用后端API的代码
                console.log('赠送解锁次数:', result.unlockCount, '给用户:', user.uid);
                this.userService.giveGift({userId: user.uid, num: result.unlockCount,adminId: this.admin.id} as Gift).subscribe({
                  next: () => {
                    this.message.success(`成功赠送 ${result.unlockCount} 次免费解锁次数`);
                    resolve(true);
                  },
                  error: (err) => {
                    this.message.error('赠送失败: ' + (err.message || '未知错误'));
                    resolve(false);
                  }
                });
              },
              nzOnCancel: () => {
                resolve(false);
              }
            });
          });
        }
        return false; // 返回false阻止弹窗关闭
      }
    });

    modal.afterClose.subscribe(result => {
      if (result) {
        this.loadData(); // 重新加载数据
      }
    });
  }

}
